@@ -298,9 +298,11 @@ USA.
 
     (define (mark! object)
       (let ((value
-	     (case (hash-table-ref/default table object 'unseen)
-	       ((unseen) 'seen)
-	       ((seen) 'shared))))
+	     (let ((value (hash-table-ref/default table object 'unseen)))
+	       (case value
+		 ((unseen) 'seen)
+		 ((seen shared) 'shared)
+		 (else (error "Invalid sharing state:" value))))))
 	(hash-table-set! table object value)
 	(eq? 'seen value)))
 
@@ -345,12 +347,7 @@ USA.
 		   (standard-print-method-name print-method object)
 		   object
 		   context
-		   (lambda (context*)
-		     (for-each (lambda (part)
-				 (*print-char #\space context*)
-				 (print-object part context*))
-			       (standard-print-method-parts print-method
-							    object))))
+		   (standard-print-method-parts print-method object))
 		  (call-print-method print-method object context))))
 	(else
 	 ((vector-ref dispatch-table
@@ -414,7 +411,6 @@ USA.
 	       (positive-fixnum ,print-number)
 	       (primitive ,print-primitive-procedure)
 	       (procedure ,print-compound-procedure)
-	       (promise ,print-promise)
 	       (ratnum ,print-number)
 	       (record ,print-record)
 	       (return-address ,print-return-address)
@@ -451,60 +447,130 @@ USA.
   (*print-string "#@" context)
   (*print-hash object context))
 
+(define (safe-car pair)
+  (map-reference-trap (lambda () (car pair))))
+
+(define (safe-cdr pair)
+  (map-reference-trap (lambda () (cdr pair))))
+
+(define (nmv-header? vector index)
+  (fix:= (ucode-type manifest-nm-vector)
+	 ((ucode-primitive primitive-type-ref 2) vector (fix:+ 1 index))))
+
+(define (nmv-header-length vector index)
+  ((ucode-primitive primitive-datum-ref 2) vector (fix:+ 1 index)))
+
+(define (safe-vector-ref vector index)
+  (map-reference-trap (lambda () (vector-ref vector index))))
+
 (define (allowed-char? char context)
   (char-in-set? char (context-char-set context)))
 
-(define (*print-with-brackets name object context procedure)
+(define (limit-print-depth context kernel)
+  (let ((context* (context-down-list context))
+	(limit (context-list-depth-limit context)))
+    (if (and limit
+	     (> (context-list-depth context*) limit))
+	(*print-string "..." context*)
+	(kernel context*))))
+
+(define (limit-print-breadth context n-printed kernel)
+  (if (let ((limit (context-list-breadth-limit context)))
+	(and limit
+	     (>= n-printed limit)))
+      (*print-string " ..." context)
+      (kernel)))
+
+(define (*general-print-items items context print-item n-printed split)
+  (let loop ((items items) (n-printed n-printed))
+    (split items
+      (lambda (item rest)
+	(limit-print-breadth context n-printed
+	  (lambda ()
+	    (if (> n-printed 0)
+		(*print-char #\space context))
+	    (print-item item context)
+	    (loop rest (+ n-printed 1))))))))
+
+(define (*print-with-brackets name object context items)
   (if (get-param:print-with-maximum-readability?)
       (*print-readable-hash object context)
-      (begin
-	(*print-string "#[" context)
-	(let ((context* (context-in-brackets context)))
-	  (if (string? name)
-	      (*print-string name context*)
-	      (print-object name context*))
-	  (if (param:print-hash-number-in-objects?)
-	      (begin
-		(*print-char #\space context*)
-		(*print-hash object context*)))
-	  (cond (procedure
-		 (procedure context*))
-		((get-param:print-with-datum?)
-		 (*print-char #\space context*)
-		 (*print-datum object context*))))
-	(*print-char #\] context))))
+      (let ((context* (context-in-brackets context)))
+	(*print-string "#[" context*)
+	(*print-items (cons*-if (if (string? name)
+				    (printing-item *print-string name)
+				    name)
+				(and (or (param:print-hash-number-in-objects?)
+					 (null? items))
+				     (printing-item *print-hash object))
+				items)
+		      context*)
+	(*print-char #\] context*))))
+
+(define (*print-items items context)
+  (*general-print-items items context *print-item 0
+    (lambda (items k)
+      (if (pair? items)
+	  (k (car items) (cdr items))))))
+
+(define (*print-item item context)
+  (cond ((printing-item? item)
+	 ((printing-item-printer item)
+	  (printing-item-object item)
+	  context))
+	((and (list? item)
+	      (any printing-item? item))
+	 (limit-print-depth context
+	   (lambda (context*)
+	     (*print-char #\( context*)
+	     (*print-items item context*)
+	     (*print-char #\) context*))))
+	(else
+	 (print-object item context))))
+
+(define-record-type <printing-item>
+    (printing-item printer object)
+    printing-item?
+  (printer printing-item-printer)
+  (object printing-item-object))
+
+(define (maybe-print-datum object)
+  (list-if (and (get-param:print-with-datum?)
+		(printing-item *print-datum object))))
+
+(define (list-if . items)
+  (remove not items))
+
+(define (cons-if car cdr)
+  (if car
+      (cons car cdr)
+      cdr))
+
+(define (cons*-if arg . args)
+  (let loop ((arg arg) (args args))
+    (if (pair? args)
+	(cons-if arg (loop (car args) (cdr args)))
+	arg)))
 
 ;;;; Printer methods
 
 (define (print-default object context)
   (let ((type (user-object-type object)))
     (case (object-gc-type object)
-      ((cell pair triple quadruple vector compiled-entry)
-       (*print-with-brackets type object context #f))
-      ((non-pointer)
-       (*print-with-brackets type object context
-         (lambda (context*)
-	   (*print-char #\space context*)
-           (*print-datum object context*))))
-      (else                             ;UNDEFINED, GC-INTERNAL
-       (*print-with-brackets type #f context
-         (lambda (context*)
-	   (*print-char #\space context*)
-           (*print-datum object context*)))))))
+      ((cell pair triple quadruple vector compiled-entry compiled-return)
+       (*print-with-brackets type object context '()))
+      (else                             ;non-pointer, undefined, gc-internal
+       (*print-with-brackets type object context (maybe-print-datum object))))))
 
 (define (user-object-type object)
   (let ((type-code (object-type object)))
     (let ((type-name (microcode-type/code->name type-code)))
       (if type-name
-          (rename-user-object-type type-name)
-          (intern
-           (string-append "undefined-type:" (number->string type-code)))))))
-
-(define (rename-user-object-type type-name)
-  (let ((entry (assq type-name renamed-user-object-types)))
-    (if entry
-        (cdr entry)
-        type-name)))
+	  (let ((entry (assq type-name renamed-user-object-types)))
+	    (if entry
+		(cdr entry)
+		type-name))
+          (string-append "undefined-type:" (number->string type-code))))))
 
 (define renamed-user-object-types
   '((access . scode-access)
@@ -560,9 +626,7 @@ USA.
   (if (get-param:print-uninterned-symbols-by-name?)
       (print-symbol-name (symbol->string symbol) context)
       (*print-with-brackets 'uninterned-symbol symbol context
-        (lambda (context*)
-	  (*print-char #\space context*)
-	  (*print-string (symbol->string symbol) context*)))))
+	(list (printing-item print-symbol-name (symbol->string symbol))))))
 
 (define (print-symbol symbol context)
   (if (keyword? symbol)
@@ -596,7 +660,7 @@ USA.
       (begin
         (*print-char #\| context)
 	(string-for-each (lambda (char)
-			   (print-string-char char context))
+			   (print-string-char char context #\|))
 			 s)
         (*print-char #\| context))))
 
@@ -615,7 +679,8 @@ USA.
 	 (*print-string (number->string (char->integer char) 16) context))
 	((context-slashify? context)
 	 (*print-string "#\\" context)
-	 (if (and (char-in-set? char char-set:normal-printing)
+	 (if (and (fix:= 0 (char-bits char))
+		  (char-in-set? char char-set:normal-printing)
 		  (not (eq? 'separator:space (char-general-category char)))
 		  (allowed-char? char context))
 	     (*print-char char context)
@@ -634,13 +699,13 @@ USA.
           (*print-char #\" context)
 	  (do ((index 0 (fix:+ index 1)))
 	      ((not (fix:< index end*)))
-	    (print-string-char (string-ref string index) context))
+	    (print-string-char (string-ref string index) context #\"))
           (if (< end* end)
               (*print-string "..." context))
           (*print-char #\" context))
       (*print-string string context)))
 
-(define (print-string-char char context)
+(define (print-string-char char context quote-char)
   (case char
     ((#\bel)
      (*print-char #\\ context)
@@ -657,13 +722,16 @@ USA.
     ((#\tab)
      (*print-char #\\ context)
      (*print-char #\t context))
-    ((#\\ #\" #\|)
+    ((#\\)
      (*print-char #\\ context)
-     (*print-char char context))
+     (*print-char #\\ context))
     (else
      (if (and (char-in-set? char char-set:normal-printing)
 	      (allowed-char? char context))
-	 (*print-char char context)
+	 (begin
+	   (if (eqv? char quote-char)
+	       (*print-char #\\ context))
+	   (*print-char char context))
 	 (begin
 	   (*print-char #\\ context)
 	   (*print-char #\x context)
@@ -679,80 +747,42 @@ USA.
           (loop (fix:- index 1))))))
 
 (define (print-vector vector context)
-  (let ((printer (named-vector-with-unparser? vector)))
-    (if printer
-	(call-print-method printer vector context)
-	(limit-print-depth context
-	  (lambda (context*)
-	    (let ((end (vector-length vector)))
-	      (if (fix:> end 0)
-		  (begin
-		    (*print-string "#(" context*)
-		    (print-object (safe-vector-ref vector 0) context*)
-		    (let loop ((index 1))
-		      (if (fix:< index end)
-			  (if (let ((limit
-				     (context-list-breadth-limit context*)))
-				(and limit
-				     (>= index limit)))
-			      (*print-string " ...)" context*)
-			      (begin
-				(*print-char #\space context*)
-				(print-object (safe-vector-ref vector index)
-					      context*)
-				(loop (fix:+ index 1))))))
-		    (*print-char #\) context*))
-		  (*print-string "#()" context*))))))))
-
-(define (safe-vector-ref vector index)
-  (if (with-absolutely-no-interrupts
-       (lambda ()
-         (object-type? (ucode-type manifest-nm-vector)
-                       (vector-ref vector index))))
-      (error "Attempt to print partially marked vector."))
-  (map-reference-trap (lambda () (vector-ref vector index))))
+  (limit-print-depth context
+    (lambda (context*)
+      (*print-string "#(" context*)
+      (let ((end (vector-length vector)))
+	(*general-print-items 0 context* print-object 0
+	  (lambda (index k)
+	    (if (< index end)
+		(if (nmv-header? vector index)
+		    ;; An embedded non-marked vector: skip over and continue.
+		    (let ((length (nmv-header-length vector index)))
+		      (k (symbol "#[non-marked section of length " length "]")
+			 (+ index 1 length)))
+		    (k (safe-vector-ref vector index)
+		       (+ index 1)))))))
+      (*print-char #\) context*))))
 
 (define (print-bytevector bytevector context)
   (limit-print-depth context
     (lambda (context*)
+      (*print-string "#u8(" context*)
       (let ((end (bytevector-length bytevector)))
-	(if (fix:> end 0)
-	    (begin
-	      (*print-string "#u8(" context*)
-	      (print-number (bytevector-u8-ref bytevector 0) context*)
-	      (let loop ((index 1))
-		(if (fix:< index end)
-		    (if (let ((limit (get-param:printer-list-breadth-limit)))
-			  (and limit
-			       (>= index limit)))
-			(*print-string " ...)" context*)
-			(begin
-			  (*print-char #\space context*)
-			  (print-number (bytevector-u8-ref bytevector index)
-					context*)
-			  (loop (fix:+ index 1))))))
-	      (*print-char #\) context*))
-	    (*print-string "#u8()" context*))))))
+	(*general-print-items 0 context* print-object 0
+	  (lambda (index k)
+	    (if (fix:< index end)
+		(k (bytevector-u8-ref bytevector index)
+		   (fix:+ index 1))))))
+      (*print-char #\) context*))))
 
 (define (print-record record context)
-  (cond ((uri? record) (print-uri record context))
-	((get-param:print-with-maximum-readability?)
-	 (*print-readable-hash record context))
-	(else
-	 (*print-with-brackets 'record record context #f))))
-
-(define (print-uri uri context)
-  (*print-string "#<" context)
-  (*print-string (uri->string uri) context)
-  (*print-string ">" context))
+  (*print-with-brackets 'record record context '()))
 
 (define (print-pair pair context)
   (cond ((prefix-pair? pair)
          => (lambda (prefix) (print-prefix-pair prefix pair context)))
         ((and (get-param:print-streams?) (stream-pair? pair))
          (print-stream-pair pair context))
-	((named-list-with-unparser? pair)
-	 => (lambda (printer) (call-print-method printer pair context)))
         (else
          (print-list pair context))))
 
@@ -761,36 +791,20 @@ USA.
     (lambda (context*)
       (*print-char #\( context*)
       (print-object (safe-car list) context*)
-      (print-tail (safe-cdr list) 2 context*)
+      (*general-print-items (safe-cdr list) context* print-object 1
+	(lambda (tail k)
+	  (cond ((datum-label tail context*)
+		 => (lambda (label)
+		      (*print-string " . " context*)
+		      (if (print-datum-label label context*)
+			  (print-object-1 tail context*))))
+		((pair? tail)
+		 (k (safe-car tail) (safe-cdr tail)))
+		((not (null? tail))
+		 (*print-string " . " context*)
+		 (print-object-1 tail context*)))))
       (*print-char #\) context*))))
 
-(define (limit-print-depth context kernel)
-  (let ((context* (context-down-list context))
-	(limit (context-list-depth-limit context)))
-    (if (and limit
-	     (> (context-list-depth-limit context*) limit))
-	(*print-string "..." context*)
-	(kernel context*))))
-
-(define (print-tail l n context)
-  (cond ((datum-label l context)
-	 => (lambda (label)
-	      (*print-string " . " context)
-	      (if (print-datum-label label context)
-                  (print-object-1 l context))))
-	((pair? l)
-	 (*print-char #\space context)
-	 (print-object (safe-car l) context)
-	 (if (let ((limit (context-list-breadth-limit context)))
-	       (and limit
-		    (>= n limit)
-		    (pair? (safe-cdr l))))
-	     (*print-string " ..." context)
-	     (print-tail (safe-cdr l) (+ n 1) context)))
-        ((not (null? l))
-         (*print-string " . " context)
-         (print-object l context))))
-
 (define (prefix-pair? object)
   (and (get-param:printer-abbreviate-quotations?)
        (pair? (safe-cdr object))
@@ -811,63 +825,38 @@ USA.
     (lambda (context*)
       (*print-char #\{ context*)
       (print-object (safe-car stream-pair) context*)
-      (print-stream-tail (safe-cdr stream-pair) 2 context*)
+      (*general-print-items (safe-cdr stream-pair) context* print-object 1
+	(lambda (tail k)
+	  (cond ((not (promise? tail))
+		 (*print-string " . " context*)
+		 (print-object tail context*))
+		((not (promise-forced? tail))
+		 (*print-string " ..." context*))
+		(else
+		 (let ((value (promise-value tail)))
+		   (cond ((empty-stream? value))
+			 ((stream-pair? value)
+			  (k (safe-car value) (safe-cdr value)))
+			 (else
+			  (*print-string " . " context*)
+			  (print-object value context*))))))))
       (*print-char #\} context*))))
-
-(define (print-stream-tail tail n context)
-  (cond ((not (promise? tail))
-         (*print-string " . " context)
-         (print-object tail context))
-        ((not (promise-forced? tail))
-         (*print-string " ..." context))
-        (else
-	 (let ((value (promise-value tail)))
-	   (cond ((empty-stream? value))
-		 ((stream-pair? value)
-		  (*print-char #\space context)
-		  (print-object (safe-car value) context)
-		  (if (let ((limit (context-list-breadth-limit context)))
-			(and limit
-			     (>= n limit)))
-		      (*print-string " ..." context)
-		      (print-stream-tail (safe-cdr value) (+ n 1) context)))
-		 (else
-		  (*print-string " . " context)
-		  (print-object value context)))))))
-
-(define (safe-car pair)
-  (map-reference-trap (lambda () (car pair))))
-
-(define (safe-cdr pair)
-  (map-reference-trap (lambda () (cdr pair))))
 
 ;;;; Procedures
 
 (define (print-compound-procedure procedure context)
   (*print-with-brackets 'compound-procedure procedure context
-    (and (get-param:print-compound-procedure-names?)
-	 (lambda-components* (procedure-lambda procedure)
-	   (lambda (name required optional rest body)
-	     required optional rest body
-	     (and (not (eq? name scode-lambda-name:unnamed))
-		  (lambda (context*)
-		    (*print-char #\space context*)
-		    (print-object name context*))))))))
+    (let ((name (scode-lambda-name (procedure-lambda procedure))))
+      (list-if (and (get-param:print-compound-procedure-names?)
+		    (not (eq? name scode-lambda-name:unnamed))
+		    name)))))
 
 (define (print-primitive-procedure procedure context)
-  (let ((print-name
-	 (lambda (context)
-	   (print-object (primitive-procedure-name procedure) context))))
-    (cond ((get-param:print-primitives-by-name?)
-	   (print-name context))
-	  ((get-param:print-with-maximum-readability?)
-	   (*print-readable-hash procedure context))
-	  (else
-	   (*print-with-brackets 'primitive-procedure #f context
-	     (lambda (context*)
-	       (*print-char #\space context*)
-	       (print-name context*)))))))
-
+  (if (get-param:print-primitives-by-name?)
+      (print-object (primitive-procedure-name procedure) context)
+      (*print-with-brackets 'primitive-procedure procedure context
+	(list (primitive-procedure-name procedure)))))
+
 (define (print-compiled-entry entry context)
   (let* ((type (compiled-entry-type entry))
          (procedure? (eq? type 'compiled-procedure))
@@ -876,101 +865,66 @@ USA.
                (compiled-code-block/manifest-closure?
                 (compiled-code-address->block entry)))))
     (*print-with-brackets (if closure? 'compiled-closure type)
-			    entry
-			    context
-      (lambda (context*)
-	(let ((name (and procedure? (compiled-procedure/name entry))))
-	  (receive (filename block-number library)
-	      (compiled-entry/filename-and-index entry)
-	    (*print-char #\space context*)
-	    (*print-char #\( context*)
-	    (if name
-		(*print-string name context*))
-	    (if filename
-		(begin
-		  (if name
-		      (*print-char #\space context*))
-		  (print-block-info filename block-number library context*)))
-	    (*print-char #\) context*)))
-	(*print-char #\space context*)
-	(*print-hex (compiled-entry/offset entry) context*)
-	(if closure?
-	    (begin
-	      (*print-char #\space context*)
-	      (*print-datum (compiled-closure->entry entry)
-			    context*)))
-	(*print-char #\space context*)
-	(*print-datum entry context*)))))
+			  entry
+			  context
+      (cons* (let ((name (and procedure? (compiled-procedure/name entry))))
+	       (cons-if (and name (printing-item *print-string name))
+			(cc-block-info (compiled-entry/block entry))))
+	     (printing-item *print-hex (compiled-entry/offset entry))
+	     (list-if (and closure?
+			   (printing-item *print-datum
+					  (compiled-closure->entry entry)))
+		      (printing-item *print-datum entry))))))
 
 (define (print-compiled-code-block block context)
   (*print-with-brackets 'compiled-code-block block context
-    (lambda (context*)
-      (receive (filename block-number library)
-	  (compiled-code-block/filename-and-index block)
-	(*print-char #\space context*)
-	(if filename
-	    (begin
-	      (*print-char #\( context*)
-	      (print-block-info filename block-number library context*)
-	      (*print-char #\) context*))))
-      (*print-char #\space context*)
-      (*print-datum block context*))))
+    (list (cc-block-info block)
+	  (list (printing-item *print-datum block)))))
 
-(define (print-block-info filename block-number library context*)
-  (print-object (pathname-name filename) context*)
-  (if block-number
-      (begin
-	(*print-char #\space context*)
-	(*print-hex block-number context*)))
-  (if (library-name? library)
-      (begin
-	(*print-char #\space context*)
-	(print-object library context*))))
+(define (cc-block-info block)
+  (receive (filename block-number library)
+      (compiled-code-block/filename-and-index block)
+    (if filename
+	(list-if (pathname-name filename)
+		 (and block-number
+		      (printing-item *print-hex block-number))
+		 (and (library-name? library)
+		      library))
+	'())))
 
 ;;;; Miscellaneous
 
 (define (print-return-address return-address context)
   (*print-with-brackets 'return-address return-address context
-    (lambda (context*)
-      (*print-char #\space context*)
-      (print-object (return-address/name return-address) context*))))
+    (list (return-address/name return-address))))
 
 (define (print-assignment assignment context)
   (*print-with-brackets 'assignment assignment context
-    (lambda (context*)
-      (*print-char #\space context*)
-      (print-object (scode-assignment-name assignment) context*))))
+    (list (scode-assignment-name assignment))))
 
 (define (print-definition definition context)
   (*print-with-brackets 'definition definition context
-    (lambda (context*)
-      (*print-char #\space context*)
-      (print-object (scode-definition-name definition) context*))))
+    (list (scode-definition-name definition))))
 
 (define (print-lambda lambda-object context)
   (*print-with-brackets 'lambda lambda-object context
-    (lambda (context*)
-      (*print-char #\space context*)
-      (print-object (scode-lambda-name lambda-object) context*))))
+    (list (scode-lambda-name lambda-object))))
 
 (define (print-variable variable context)
   (*print-with-brackets 'variable variable context
-    (lambda (context*)
-      (*print-char #\space context*)
-      (print-object (scode-variable-name variable) context*))))
+    (list (scode-variable-name variable))))
 
 (define (print-number object context)
   (*print-string (number->string
 		  object
 		  (let ((prefix
 			 (lambda (prefix limit radix)
-			   (if (exact-rational? object)
-			       (begin
-				 (if (not (and (exact-integer? object)
-					       (< (abs object) limit)))
-				     (*print-string prefix context))
-				 radix)
-			       10))))
+			   (if (not (or (and (flo:flonum? object)
+					     (not (flo:finite? object)))
+					(and (exact-integer? object)
+					     (< (abs object) limit))))
+			       (*print-string prefix context))
+			   radix)))
 		    (case (get-param:printer-radix)
 		      ((2) (prefix "#b" 2 2))
 		      ((8) (prefix "#o" 8 8))
@@ -984,36 +938,19 @@ USA.
       (print-floating-vector flonum context)))
 
 (define (print-floating-vector v context)
-  (let ((length ((ucode-primitive floating-vector-length) v)))
-    (*print-with-brackets "floating-vector" v context
-      (and (not (zero? length))
-           (lambda (context*)
-             (let ((limit
-		    (let ((limit (get-param:printer-list-breadth-limit)))
-		      (if limit
-			  (min length limit)
-			  length))))
-               (*print-char #\space context*)
-	       (print-flonum ((ucode-primitive floating-vector-ref) v 0)
-			       context*)
-               (do ((i 1 (+ i 1)))
-                   ((>= i limit))
-                 (*print-char #\space context*)
-                 (print-flonum ((ucode-primitive floating-vector-ref) v i)
-				 context*))
-               (if (< limit length)
-                   (*print-string " ..." context*))))))))
+  (*print-with-brackets 'floating-vector v context
+    (map (lambda (index)
+	   (printing-item print-number (flo:vector-ref v index)))
+	 (iota (flo:vector-length v)))))
 
 (define (print-entity entity context)
 
   (define (plain name)
-    (*print-with-brackets name entity context #f))
+    (*print-with-brackets name entity context '()))
 
   (define (named-arity-dispatched-procedure name)
     (*print-with-brackets 'arity-dispatched-procedure entity context
-      (lambda (context*)
-        (*print-char #\space context*)
-	(*print-string name context*))))
+      (list (printing-item *print-string name))))
 
   (cond ((continuation? entity)
          (plain 'continuation))
@@ -1025,32 +962,15 @@ USA.
                        (compiled-procedure? proc)
                        (compiled-procedure/name proc))
                   => named-arity-dispatched-procedure)
-                 (else (plain 'arity-dispatched-procedure)))))
-        ((get-param:print-with-maximum-readability?)
-         (*print-readable-hash entity context))
-        (else (plain 'entity))))
-
-(define (print-promise promise context)
-  (*print-with-brackets 'promise promise context
-    (if (promise-forced? promise)
-	(lambda (context*)
-	  (*print-string " (evaluated) " context*)
-	  (print-object (promise-value promise) context*))
-	(lambda (context*)
-	  (*print-string " (unevaluated)" context*)
-	  (if (get-param:print-with-datum?)
-	      (begin
-		(*print-char #\space context*)
-		(*print-datum promise context*)))))))
+                 (else
+		  (plain 'arity-dispatched-procedure)))))
+        (else
+	 (plain 'entity))))
 
 (define (print-tagged-object object context)
   (*print-with-brackets 'tagged-object object context
-    (lambda (context*)
-      (*print-char #\space context*)
-      (print-object (let ((tag (%tagged-object-tag object)))
-		       (if (dispatch-tag? tag)
-			   (dispatch-tag-name tag)
-			   tag))
-		     context*)
-      (*print-char #\space context*)
-      (print-object (%tagged-object-datum object) context*))))
+    (list (let ((tag (%tagged-object-tag object)))
+	    (if (dispatch-tag? tag)
+		(dispatch-tag-print-name tag)
+		tag))
+	  (%tagged-object-datum object))))
